@@ -1,8 +1,10 @@
 use super::ValidationRule;
+use crate::ast::{
+    visit_document, OperationVisitor, OperationVisitorContext, SchemaDocumentExtension,
+    TypeDefinitionExtension,
+};
 use crate::static_graphql::query::*;
-use crate::static_graphql::schema::TypeDefinition;
 use crate::validation::utils::{ValidationError, ValidationErrorContext};
-use crate::{ast::QueryVisitor, validation::utils::ValidationContext};
 
 /// Fragments on composite type
 ///
@@ -13,67 +15,68 @@ use crate::{ast::QueryVisitor, validation::utils::ValidationContext};
 /// https://spec.graphql.org/draft/#sec-Fragments-On-Composite-Types
 pub struct FragmentsOnCompositeTypes;
 
-impl<'a> QueryVisitor<ValidationErrorContext<'a>> for FragmentsOnCompositeTypes {
-    fn enter_inline_fragment(
-        &self,
-        _node: &InlineFragment,
-        _visitor_context: &mut ValidationErrorContext<'a>,
-    ) {
-        if let Some(TypeCondition::On(type_condition)) = &_node.type_condition {
-            let gql_type = _visitor_context
-                .ctx
-                .find_schema_definition_by_name(type_condition.to_owned());
+impl FragmentsOnCompositeTypes {
+    pub fn new() -> Self {
+        FragmentsOnCompositeTypes
+    }
+}
 
-            if let Some(gql_type) = gql_type {
-                match gql_type {
-                    TypeDefinition::Object(_)
-                    | TypeDefinition::Interface(_)
-                    | TypeDefinition::Union(_) => {}
-                    _ => _visitor_context.report_error(ValidationError {
-                        locations: vec![_node.position],
+impl<'a> OperationVisitor<'a, ValidationErrorContext> for FragmentsOnCompositeTypes {
+    fn enter_inline_fragment(
+        &mut self,
+        visitor_context: &mut OperationVisitorContext,
+        user_context: &mut ValidationErrorContext,
+        inline_fragment: &InlineFragment,
+    ) {
+        if let Some(TypeCondition::On(type_condition)) = &inline_fragment.type_condition {
+            if let Some(gql_type) = visitor_context.schema.type_by_name(type_condition) {
+                if !gql_type.is_composite_type() {
+                    user_context.report_error(ValidationError {
+                        locations: vec![inline_fragment.position],
                         message: format!(
                             "Fragment cannot condition on non composite type \"{}\".",
                             type_condition
                         ),
-                    }),
+                    })
                 }
             }
         }
     }
 
     fn enter_fragment_definition(
-        &self,
-        _node: &FragmentDefinition,
-        _visitor_context: &mut ValidationErrorContext<'a>,
+        &mut self,
+        visitor_context: &mut OperationVisitorContext,
+        user_context: &mut ValidationErrorContext,
+        fragment_definition: &FragmentDefinition,
     ) {
-        let TypeCondition::On(type_condition) = &_node.type_condition;
+        let TypeCondition::On(type_condition) = &fragment_definition.type_condition;
 
-        if let Some(gql_type) = _visitor_context
-            .ctx
-            .find_schema_definition_by_name(type_condition.to_owned())
-        {
-            match gql_type {
-                TypeDefinition::Object(_)
-                | TypeDefinition::Interface(_)
-                | TypeDefinition::Union(_) => {}
-                _ => _visitor_context.report_error(ValidationError {
-                    locations: vec![_node.position],
+        if let Some(gql_type) = visitor_context.schema.type_by_name(type_condition) {
+            if !gql_type.is_composite_type() {
+                user_context.report_error(ValidationError {
+                    locations: vec![fragment_definition.position],
                     message: format!(
                         "Fragment \"{}\" cannot condition on non composite type \"{}\".",
-                        _node.name, type_condition
+                        fragment_definition.name, type_condition
                     ),
-                }),
+                })
             }
         }
     }
 }
 
 impl ValidationRule for FragmentsOnCompositeTypes {
-    fn validate<'a>(&self, ctx: &ValidationContext) -> Vec<ValidationError> {
-        let mut error_context = ValidationErrorContext::new(ctx);
-        self.visit_document(&ctx.operation.clone(), &mut error_context);
-
-        error_context.errors
+    fn validate<'a>(
+        &self,
+        ctx: &'a mut OperationVisitorContext,
+        error_collector: &mut ValidationErrorContext,
+    ) {
+        visit_document(
+            &mut FragmentsOnCompositeTypes::new(),
+            &ctx.operation,
+            ctx,
+            error_collector,
+        );
     }
 }
 
@@ -82,10 +85,11 @@ fn object_is_valid_fragment_type() {
     use crate::validation::test_utils::*;
 
     let mut plan = create_plan_from_rule(Box::new(FragmentsOnCompositeTypes {}));
-    let errors = test_operation_without_schema(
+    let errors = test_operation_with_schema(
         "fragment validFragment on Dog {
           barks
         }",
+        TEST_SCHEMA,
         &mut plan,
     );
 
@@ -97,10 +101,11 @@ fn interface_is_valid_fragment_type() {
     use crate::validation::test_utils::*;
 
     let mut plan = create_plan_from_rule(Box::new(FragmentsOnCompositeTypes {}));
-    let errors = test_operation_without_schema(
+    let errors = test_operation_with_schema(
         "fragment validFragment on Pet {
           name
         }",
+        TEST_SCHEMA,
         &mut plan,
     );
 
@@ -112,12 +117,13 @@ fn object_is_valid_inline_fragment_type() {
     use crate::validation::test_utils::*;
 
     let mut plan = create_plan_from_rule(Box::new(FragmentsOnCompositeTypes {}));
-    let errors = test_operation_without_schema(
+    let errors = test_operation_with_schema(
         "fragment validFragment on Pet {
           ... on Dog {
             barks
           }
         }",
+        TEST_SCHEMA,
         &mut plan,
     );
 
@@ -129,12 +135,13 @@ fn interface_is_valid_inline_fragment_type() {
     use crate::validation::test_utils::*;
 
     let mut plan = create_plan_from_rule(Box::new(FragmentsOnCompositeTypes {}));
-    let errors = test_operation_without_schema(
+    let errors = test_operation_with_schema(
         "fragment validFragment on Mammal {
           ... on Canine {
             name
           }
         }",
+        TEST_SCHEMA,
         &mut plan,
     );
 
@@ -146,12 +153,13 @@ fn inline_fragment_without_type_is_valid() {
     use crate::validation::test_utils::*;
 
     let mut plan = create_plan_from_rule(Box::new(FragmentsOnCompositeTypes {}));
-    let errors = test_operation_without_schema(
+    let errors = test_operation_with_schema(
         "fragment validFragment on Pet {
           ... {
             name
           }
         }",
+        TEST_SCHEMA,
         &mut plan,
     );
 
@@ -163,10 +171,11 @@ fn union_is_valid_fragment_type() {
     use crate::validation::test_utils::*;
 
     let mut plan = create_plan_from_rule(Box::new(FragmentsOnCompositeTypes {}));
-    let errors = test_operation_without_schema(
+    let errors = test_operation_with_schema(
         "fragment validFragment on CatOrDog {
           __typename
         }",
+        TEST_SCHEMA,
         &mut plan,
     );
 
