@@ -1,7 +1,7 @@
 use super::ValidationRule;
+use crate::ast::{visit_document, OperationVisitor, OperationVisitorContext};
 use crate::static_graphql::query::*;
 use crate::validation::utils::{ValidationError, ValidationErrorContext};
-use crate::{ast::QueryVisitor, validation::utils::ValidationContext};
 
 /// Lone Anonymous Operation
 ///
@@ -11,9 +11,20 @@ use crate::{ast::QueryVisitor, validation::utils::ValidationContext};
 /// https://spec.graphql.org/draft/#sec-Lone-Anonymous-Operation
 pub struct LoneAnonymousOperation;
 
-impl<'a> QueryVisitor<ValidationErrorContext<'a>> for LoneAnonymousOperation {
-    fn enter_document(&self, _node: &Document, visitor_context: &mut ValidationErrorContext<'a>) {
-        let operations_count = _node
+impl LoneAnonymousOperation {
+    pub fn new() -> Self {
+        LoneAnonymousOperation
+    }
+}
+
+impl<'a> OperationVisitor<'a, ValidationErrorContext> for LoneAnonymousOperation {
+    fn enter_document(
+        &mut self,
+        _: &mut OperationVisitorContext,
+        user_context: &mut ValidationErrorContext,
+        document: &Document,
+    ) {
+        let operations_count = document
             .definitions
             .iter()
             .filter(|n| match n {
@@ -25,11 +36,11 @@ impl<'a> QueryVisitor<ValidationErrorContext<'a>> for LoneAnonymousOperation {
             })
             .count();
 
-        for definition in &_node.definitions {
+        for definition in &document.definitions {
             match definition {
                 Definition::Operation(OperationDefinition::SelectionSet(_)) => {
                     if operations_count > 1 {
-                        visitor_context.report_error(ValidationError {
+                        user_context.report_error(ValidationError {
                             message: "This anonymous operation must be the only defined operation."
                                 .to_string(),
                             locations: vec![],
@@ -38,7 +49,7 @@ impl<'a> QueryVisitor<ValidationErrorContext<'a>> for LoneAnonymousOperation {
                 }
                 Definition::Operation(OperationDefinition::Query(query)) => {
                     if query.name == None && operations_count > 1 {
-                        visitor_context.report_error(ValidationError {
+                        user_context.report_error(ValidationError {
                             message: "This anonymous operation must be the only defined operation."
                                 .to_string(),
                             locations: vec![query.position.clone()],
@@ -47,7 +58,7 @@ impl<'a> QueryVisitor<ValidationErrorContext<'a>> for LoneAnonymousOperation {
                 }
                 Definition::Operation(OperationDefinition::Mutation(mutation)) => {
                     if mutation.name == None && operations_count > 1 {
-                        visitor_context.report_error(ValidationError {
+                        user_context.report_error(ValidationError {
                             message: "This anonymous operation must be the only defined operation."
                                 .to_string(),
                             locations: vec![mutation.position.clone()],
@@ -56,7 +67,7 @@ impl<'a> QueryVisitor<ValidationErrorContext<'a>> for LoneAnonymousOperation {
                 }
                 Definition::Operation(OperationDefinition::Subscription(subscription)) => {
                     if subscription.name == None && operations_count > 1 {
-                        visitor_context.report_error(ValidationError {
+                        user_context.report_error(ValidationError {
                             message: "This anonymous operation must be the only defined operation."
                                 .to_string(),
                             locations: vec![subscription.position.clone()],
@@ -70,11 +81,17 @@ impl<'a> QueryVisitor<ValidationErrorContext<'a>> for LoneAnonymousOperation {
 }
 
 impl ValidationRule for LoneAnonymousOperation {
-    fn validate<'a>(&self, ctx: &ValidationContext) -> Vec<ValidationError> {
-        let mut error_context = ValidationErrorContext::new(ctx);
-        self.visit_document(&ctx.operation.clone(), &mut error_context);
-
-        error_context.errors
+    fn validate<'a>(
+        &self,
+        ctx: &'a mut OperationVisitorContext,
+        error_collector: &mut ValidationErrorContext,
+    ) {
+        visit_document(
+            &mut LoneAnonymousOperation::new(),
+            &ctx.operation,
+            ctx,
+            error_collector,
+        );
     }
 }
 
@@ -83,10 +100,11 @@ fn no_operations() {
     use crate::validation::test_utils::*;
 
     let mut plan = create_plan_from_rule(Box::new(LoneAnonymousOperation {}));
-    let errors = test_operation_without_schema(
+    let errors = test_operation_with_schema(
         "fragment fragA on Type {
           field
         }",
+        TEST_SCHEMA,
         &mut plan,
     );
 
@@ -98,10 +116,11 @@ fn one_anon_operation() {
     use crate::validation::test_utils::*;
 
     let mut plan = create_plan_from_rule(Box::new(LoneAnonymousOperation {}));
-    let errors = test_operation_without_schema(
+    let errors = test_operation_with_schema(
         "{
           field
         }",
+        TEST_SCHEMA,
         &mut plan,
     );
 
@@ -113,13 +132,14 @@ fn mutiple_named() {
     use crate::validation::test_utils::*;
 
     let mut plan = create_plan_from_rule(Box::new(LoneAnonymousOperation {}));
-    let errors = test_operation_without_schema(
+    let errors = test_operation_with_schema(
         "query Foo {
           field
         }
         query Bar {
           field
         }",
+        TEST_SCHEMA,
         &mut plan,
     );
 
@@ -131,13 +151,14 @@ fn anon_operation_with_fragment() {
     use crate::validation::test_utils::*;
 
     let mut plan = create_plan_from_rule(Box::new(LoneAnonymousOperation {}));
-    let errors = test_operation_without_schema(
+    let errors = test_operation_with_schema(
         "{
           ...Foo
         }
         fragment Foo on Type {
           field
         }",
+        TEST_SCHEMA,
         &mut plan,
     );
 
@@ -149,13 +170,14 @@ fn multiple_anon_operations() {
     use crate::validation::test_utils::*;
 
     let mut plan = create_plan_from_rule(Box::new(LoneAnonymousOperation {}));
-    let errors = test_operation_without_schema(
+    let errors = test_operation_with_schema(
         "{
           fieldA
         }
         {
           fieldB
         }",
+        TEST_SCHEMA,
         &mut plan,
     );
 
@@ -175,13 +197,14 @@ fn anon_operation_with_mutation() {
     use crate::validation::test_utils::*;
 
     let mut plan = create_plan_from_rule(Box::new(LoneAnonymousOperation {}));
-    let errors = test_operation_without_schema(
+    let errors = test_operation_with_schema(
         "{
           fieldA
         }
         mutation Foo {
           fieldB
         }",
+        TEST_SCHEMA,
         &mut plan,
     );
 
@@ -198,13 +221,14 @@ fn anon_operation_with_subscription() {
     use crate::validation::test_utils::*;
 
     let mut plan = create_plan_from_rule(Box::new(LoneAnonymousOperation {}));
-    let errors = test_operation_without_schema(
+    let errors = test_operation_with_schema(
         "{
           fieldA
         }
         subscription Foo {
           fieldB
         }",
+        TEST_SCHEMA,
         &mut plan,
     );
 
